package com.perforce.hws.deployment;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perforce.hws.util.StreamUtils;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;

/**
 * Helpers to identify OS and architecture, etc, based on system properties.
 */
public interface OsHelpers extends StreamUtils {

	/** Class specific logger class. */
    Logger LOGGER = LoggerFactory.getLogger(OsHelpers.class);

    /**
     * Check for MacOS/X, operating system name starts with "Mac OS X".
     * 
     * @return true or false
     */
    default boolean isOSX() {
        return osName().startsWith("Mac OS X");
    }

    /**
     * Check for Windows, operating system name starts with "Windows".
     * 
     * @return true or false
     */
    default boolean isWindows() {
        return osName().startsWith("Windows");
    }

    /**
     * Check for Windows, operating system name starts with "Windows".
     * 
     * @return true or false
     */
    default boolean isLinux() {
        return osName().startsWith("Linux");
    }

    /**
     * Get what Java thinks the os name is, using os.name property.
     * 
     * @return String - the os name
     */
    default String osName() {
        return System.getProperty("os.name");
    }

    /**
     * Get what Java thinks architecture is, using os.arch property.
     * 
     * @return e.g., "x86_64"
     */
    default String osArch() {
        return System.getProperty("os.arch");
    }

    /**
     * Get the system io disposal point.
     * 
     * @return e.g., "/dev/null"
     */
    default String nullPath() {
        if (isWindows()) {
            return "nul";
        } else {
            return "/dev/null";
        }
    }

    /**
     * Identify CentOS flavours of Linux, this looks for and examines
     * the redhat release details.
     * 
     * @return true or false
     * @throws IOException if files do not exist
     */
    default boolean isCentOS() throws IOException {
        return hasRedhatRelease() && loadRedhatRelease().contains("CentOS");
    }

    /**
     * Identify whether this is a RedHat flavour of Linux. Determined by
     * whether the redhat-release file exists.
     * 
     * @return True or false
     */
    default boolean hasRedhatRelease() {
        return Files.exists(Paths.get("/etc/redhat-release"));
    }

    /**
     * Get the flavour of RedHat Linux from the release file.
     * 
     * @return String representation of the file contents.
     * @throws IOException If the file cannot be read.
     */
    default String loadRedhatRelease() throws IOException {
        return new String(Files.readAllBytes(Paths.get("/etc/redhat-release")));
    }

    /**
     * Identify whether this is an Ubuntu flavour Linux system. Determined
     * by the exit status and output from the lsb_release command.
     * @return true of false
     * @throws IOException If the lsb_command cannot be run
     * @throws InterruptedException If the lsb_release command times out
     */
    default boolean isUbuntu() throws IOException, InterruptedException {
        return hasLsbRelease() && loadLsbRelease("-s -i").contains("Ubuntu");
    }

    /**
     * Get the short version of the distributor id; i.e. ubuntu
     * @return String value of the distributor id value
     * @throws IOException If the lsb_command cannot be run
     * @throws InterruptedException If the lsb_command times out.
     */
    default String loadUbuntuVersion() throws IOException, InterruptedException {
        return loadLsbRelease("-r -s").trim();
    }

    /**
     * Identify whether a platfrom supports lsb_release.
     * @return true or false
     * @throws IOException If the command cannot be run.
     * @throws InterruptedException If the lsb_command times out.
     */
    default boolean hasLsbRelease() throws IOException, InterruptedException {
        try {
            return Runtime.getRuntime().exec("lsb_release").waitFor() == 0;
        } catch (IOException e) {
            LOGGER.debug("Treating IOException for lsb_release as a "
                    + "'command not found' scenario", e);
            return false;
        }
    }

    /**
     * Run lsb_release and collect the output.
     * @param args flags to be passed to lsb_release
     * @return The command output as a string
     * @throws IOException If the command cannot be run.
     * @throws InterruptedException If the lsb_command times out.
     */
    default String loadLsbRelease(String args)
    		throws IOException, InterruptedException {
        Process p = Runtime.getRuntime().exec("lsb_release " + args);
        p.waitFor();
        if (p.exitValue() == 0) {
            return new String(readAll(p.getInputStream()));
        } else {
            throw new IllegalStateException("lsb_release failed with exit "
                    + "code: " + p.exitValue());
        }
    }
}
