package com.perforce.hws.deployment;

import java.nio.file.Path;
import java.nio.file.Paths;

/**
 * Created by tristan on 3/31/16.
 */
public interface PathConventions {

    /**
     * Our shell launcher should be configured to execute with the current
     * directory context set to the installation directory.
     * <p>
     * NOTE: the install4j launchers set this working directory for us.
     *
     * @return The absolute path to the installation directory, e.g.,
     * /opt/perforce/helix-web-services, or wherever the user installed it.
     */
    default Path loadInstallDir() {
        return Paths.get(System.getProperty("user.dir"))
        		.normalize().toAbsolutePath();
    }
    
    /**
     * Location of the log directory.
     *
     * @return the path
     */
    default Path logDirectory() {
    	return Paths.get(loadInstallDir().toString(), "log");
    }
    
    /**
     * Location of the tmp directory.
     *
     * @return the path
     */
    default Path tmpDirectory() {
    	return Paths.get(loadInstallDir().toString(), "tmp");
    }

    /**
     * Location of the launchd configuration for OS X.
     *
     * @return the path
     */
    default Path plistPath() {
        return Paths.get("/Library/LaunchDaemons/"
        		+ "com.perforce.HelixWebServices.plist");
    }

    /**
     * Inits the d path.
     *
     * @return the path
     */
    default Path initDPath() {
        return Paths.get("/etc/init.d/helix-ws");
    }
    
    /**
     * Keystore path.
     * @return the path
     */
    default Path keystorePath() {
        return Paths.get(loadInstallDir().toString(), "keystore.jks");
    }

    /**
     * Local cert path.
     * @return the path
     */
    default Path localCertPath() {
        return Paths.get(loadInstallDir().toString(), "localhost.cer");
    }

    /**
     * Truststore path.
     * @return the path
     */
    default Path truststorePath() {
        return Paths.get(loadInstallDir().toString(), "truststore.jks");
    }
    
    /**
     * Config path.
     * @return the path
     */
    default Path configPath() {
        return Paths.get(loadInstallDir().toString(),
        		"etc", "helix-web-services.conf");
    }
    
    /**
     * P4 config dir path.
     * @return the path
     */
    default Path p4ConfigDirPath() {
        return Paths.get(loadInstallDir().toString(), "etc", "p4d");
    }
    
    /**
     * The location of the example p4d configuration file.
     * @return the currently configuration file path
     */
    default Path localP4dConfigPath() {
    	return Paths.get(loadInstallDir().toString(), "etc", "p4d", "example");
    }
    
    /**
     * Sbin path.
     *
     * @return the path
     */
    default Path sbinPath() {
    	return Paths.get(loadInstallDir().toString(), "sbin", "helix-ws");
    }
    
    /**
     * Gets the sc path.
     *
     * @return the sc path
     */
    default Path scPath() {
        return Paths.get(System.getenv("windir"), "system32", "sc.exe");
    }
    
    /**
     * Windows launcher path.
     *
     * @return the path
     */
    default Path windowsLauncherPath() {
        return Paths.get(loadInstallDir().toString(), "sbin", "helix-ws.exe");
    }
}
