package com.perforce.hws.deployment;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perforce.hws.util.ProcessHelpers;
import com.perforce.hws.util.StreamUtils;

import java.io.IOException;
import java.nio.file.Paths;

/**
 * Methods to create system user accounts on linux systems.
 */
public interface UserUtils extends OsHelpers, StreamUtils, ProcessHelpers {

    Logger logger = LoggerFactory.getLogger(UserUtils.class);

    /**
     * The username we typically run the HWS process as on Unices.
     */
    default String unixUsername() {
        return "perforce";
    }

    default boolean userExistsViaGetent(String username) throws InterruptedException {
        try {
            String passwd = execOutOrFail("getent", "passwd", username);
            logger.debug("passwd entry: {}", passwd);
            return true;
        } catch (IllegalStateException | IOException e) {
            logger.debug("ignoring error", e);
            return false;
        }
    }

    default boolean userExistsViaDscl(String username) throws InterruptedException {
        try {
            String dscl = execOutOrFail("dscl", ".", "-read", "/Users/" + username);
            logger.debug("dscl entry: {}", dscl);
            return true;
        } catch (IOException | IllegalStateException e) {
            logger.debug("Ignoring exception", e);
            return false;
        }
    }

    default void createLinuxUser(String username, String installDir)
            throws IOException, InterruptedException {

        String homeDir = Paths.get(installDir, "home", username).toString();

        if (isCentOS()) {
            createUserWithUseradd(username, homeDir);
        } else {
            createUserWithAdduser(username, homeDir);
        }
    }

    default void createUserWithUseradd(String username, String homeDir)
            throws IOException, InterruptedException {
        logger.info("Creating account {} with home dir {} (via useradd)", username, homeDir);
        execOrFail("useradd", "-r", "--shell", "/bin/bash", "--home", homeDir,
                username);
    }

    default void createUserWithAdduser(String username, String homeDir)
            throws IOException, InterruptedException {
        logger.info("Creating account {} with home dir {} (via adduser)", username, homeDir);
        execOrFail("adduser", "--system", "--group", "--home", homeDir,
                "--shell", "/bin/bash", username);
    }

    default void createUserWithDscl(String username, String homeDir)
            throws IOException, InterruptedException {
        logger.info("Creating account {} with home dir {} (via dscl)", username, homeDir);
        execOrFail("dscl", ".", "-create", "/Groups/" + username);
        execOrFail("dscl", ".", "-create", "/Groups/" + username, "gid", "20999");
        execOrFail("dscl", ".", "-create", "/Users/" + username);
        execOrFail("dscl", ".", "-create", "/Users/" + username,
                "NFSHomeDirectory", homeDir);
        execOrFail("dscl", ".", "-create", "/Users/" + username,
                "UniqueID", "20999");
        execOrFail("dscl", ".", "-create", "/Users/" + username,
                "PrimaryGroupID", "20999");
    }

    default void removeUserAndGroupViaDscl(String name)
            throws IOException, InterruptedException {
        logger.info("Removing user and group {}", name);
        execOrFail("dscl", ".", "-delete", "/Groups/" + name);
        execOrFail("dscl", ".", "-delete", "/Users/" + name);
    }
}
