package com.perforce.hws.deployment.deployers;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.UncheckedIOException;
import java.nio.file.Files;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.perforce.hws.deployment.FileSystemHelpers;
import com.perforce.hws.deployment.JWTKeyGeneration;
import com.perforce.hws.deployment.PathConventions;
import com.perforce.hws.deployment.UserUtils;
import com.perforce.hws.util.FilesUtils;

/**
 * Generic class for all OS specific deployers to extend.
 * Copes with operations that are OS agnostic. 
 */
public abstract class Deployer implements 
		IDeployer,
		PathConventions,
		UserUtils,
		FilesUtils,
		FileSystemHelpers,
		JWTKeyGeneration {
	
	/** The logger. */
	private static final Logger LOGGER
		= LoggerFactory.getLogger(Deployer.class);
		
	/** The install dir. */
	private final String installDir = loadInstallDir().toString();

	/**
	 * Gets the install dir.
	 * @return the install dir
	 */
	protected String getInstallDir() {
		return installDir;
	}

	/* (non-Javadoc)
	 * @see com.perforce.hws.deployment.deployers.IDeployer#postUninstall()
	 */
	@Override
	public abstract void postUninstall() throws IOException;
	
	/* (non-Javadoc)
	 * @see com.perforce.hws.deployment.deployers.IDeployer#ensurePaths()
	 */
	@Override
	public void ensureDirectories() throws IOException {
		ensureDirectoryExists(loadInstallDir());
		ensureDirectoryExists(p4ConfigDirPath());
	}

	/**
	 * Creates the trust files.
	 *
	 * @throws IOException Signals that an I/O exception has occurred.
	 * @throws InterruptedException the interrupted exception
	 */
	protected void createTrustFiles() throws IOException, InterruptedException {
        if (!Files.exists(keystorePath())) {
            execOrFail("keytool", "-genkeypair",
                    "-alias", "localhost",
                    "-keyalg", "RSA",
                    "-validity", "365",
                    "-keysize", "2048",
                    "-keypass", "changeit",
                    "-storepass", "changeit",
                    "-keystore", keystorePath()
                    	.toAbsolutePath().toString(),
                    "-dname", "CN=localhost,OU=MyOrgUnit,O=MyOrg,"
                    	+ "L=Somewhere,S=State,C=US");

            execOrFail("keytool", "-exportcert",
                    "-keystore", keystorePath()
                    	.toAbsolutePath().toString(),
                    "-alias", "localhost",
                    "-storepass", "changeit",
                    "-file", localCertPath().toAbsolutePath().toString()
            );

            execOrFail("keytool", "-importcert",
                    "-keystore", truststorePath()
                    	.toAbsolutePath().toString(),
                    "-alias", "localhost",
                    "-storepass", "changeit",
                    "-file", localCertPath().toAbsolutePath().toString(),
                    "-noprompt");
        }
    }
	
	/**
	 * Determine if the config file has edits.
	 *
	 * @return true, if edited.
	 */
	protected boolean configHasNoEdits() {
        try {
            String first = Files.lines(configPath())
            			.findFirst().orElse("");
            return first.contains("NOEDITS");
        } catch (IOException io) {
            throw new UncheckedIOException(io);
        }
    }
	
	/**
	 * Write out an example p4d configuration file, if one does not already
	 * exist.
	 * 
	 * This file comprises an introduction with a section for each of the
	 * possible file entries, each section concludes with an example value
	 * for the entry. All lines are comments, to prevent unwanted side 
	 * effects.
	 * 
	 * @throws IOException If there is a problem writing the file
	 */
	protected void createExampleConfig() throws IOException {
		File example = new File(localP4dConfigPath().toString());
		if (example.exists()) {
			LOGGER.info("Leaving existing example configuration alone, at "
        		+ example.getName());
		} else {
			try (BufferedWriter exampleFileWriter = 
						new BufferedWriter(new FileWriter(example, true))) {
				for (String line: new String[]{
					"#",
					"# Example helix-web-services Helix server configuration file",
					"# ==========================================================",
					"#",
					"# This file contains configuration values for a connection to",
					"# a Perforce Helix server within a helix-web-services",
					"# deployment.",
					"#",
					"# To activate the configuration uncomment the values in this",
					"# file, or copy it to a new file first.",
					"#",
					"# The unique identifier of the Perforce Helix server within",
					"# the scope of this helix-web-services deployment.",
					"#id: localhost",
					"#",
					"# The displayed name of the Perforce Helix server within",
					"# the scope of this helix-web-services deployment.",
					"#name: localhost",
					"#",
					"# The description of this Perforce Helix server",
					"#description: Local p4d server",
					"#",
					"# The host and port for communicating with the Perforce Helix",
					"# server.",
					"#P4PORT: localhost:1666",
					"#",
					"# The client charset used when transferring data to and from",
					"# the Perforce Helix server.",
					"#P4CHARSET: utf8"
				}) {
					exampleFileWriter.write(line);
					exampleFileWriter.newLine();
				}
			}
			LOGGER.info("Created example P4D configuration " + example.getName());
		}
	}
}
