package com.perforce.hws.deployment.deployers;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.StandardOpenOption;
import java.util.stream.Collectors;

/**
 * Deployer for Windows.
 */
public class Windows extends Deployer {

    /**
     * The Constant SERVICE.
     */
    private static final String SERVICE = "helix-ws";

    /* (non-Javadoc)
     * @see com.perforce.hws.deployment.deployers.Deployer#configure()
     */
    @Override
    public void configure() throws InterruptedException, IOException {
    	ensureDirectories();
        createTrustFiles();
        // Create /opt/perforce/helix-web-services/etc/example
        createExampleConfig();
        // Register with sc.exe
        execOutOrFail(scPath().toString(), "create", SERVICE,
                "binPath=" + windowsLauncherPath().toAbsolutePath().toString());
        reconfigureWindowsDefaults();
    }

    /* (non-Javadoc)
     * @see com.perforce.hws.deployment.deployers.Deployer#uninstall()
     */
    @Override
    public void uninstall(final boolean inInstaller)
    		throws IOException, InterruptedException {
        if (isActiveViaSc()) {
            shutdownViaSc();
            deleteViaSc();
        }
    }

    /**
     * Reconfigure windows defaults.
     *
     * @throws IOException          Signals that an I/O exception has occurred.
     * @throws InterruptedException the interrupted exception
     */
    private void reconfigureWindowsDefaults()
            throws IOException, InterruptedException {
        if (configHasNoEdits()) {
            String converted = Files.lines(configPath())
                    .map(l -> {
                        if (l.contains("NOEDITS")) {
                            return "";
                        } else if (l.contains("# HWS_PORT: 9000")) {
                            return "HWS_PORT: 443";
                        } else if (l.contains("# ENABLE_HTTPS: false")) {
                            return "ENABLE_HTTPS: true";
                        } else if (l.contains("# KEYSTORE_FILE:")) {
                            return "KEYSTORE_FILE: "
                                    + keystorePath()
                                    .toAbsolutePath().toString();
                        } else if (l.contains("# KEYSTORE_PASSWORD:")) {
                            return "KEYSTORE_PASSWORD: changeit";
                        } else if (l.contains("# JWT_SIGNING_KEY:")) {
                            return "JWT_SIGNING_KEY: " + createJWTSigningKey();
                        } else if (l.contains("# TRUSTSTORE_FILE:")) {
                            return "TRUSTSTORE_FILE: "
                                    + truststorePath()
                                    .toAbsolutePath().toString();
                        } else if (l.contains("# TRUSTSTORE_PASSWORD:")) {
                            return "TRUSTSTORE_PASSWORD: changeit";
                        } else if (l.contains("# P4DCONFIGDIR:")) {
                            return "P4DCONFIGDIR: "
                                    + p4ConfigDirPath()
                                    .toAbsolutePath().toString();
                        } else {
                            return l;
                        }
                    })
                    .collect(Collectors.joining(System.lineSeparator()));
            Files.write(configPath(),
                    converted.getBytes(), StandardOpenOption.TRUNCATE_EXISTING);
        }
    }

    /**
     * Shutdown via sc.
     *
     * @throws IOException          Signals that an I/O exception has occurred.
     * @throws InterruptedException the interrupted exception
     */
    private void shutdownViaSc() throws IOException, InterruptedException {
        execOutOrFail(scPath().toString(), "stop", SERVICE);
    }

    /**
     * Delete via sc.
     *
     * @throws IOException          Signals that an I/O exception has occurred.
     * @throws InterruptedException the interrupted exception
     */
    private void deleteViaSc() throws IOException, InterruptedException {
        execOutOrFail(scPath().toString(), "delete", SERVICE);
    }

    /**
     * Checks if is active via sc.
     *
     * @return true, if is active via sc
     * @throws IOException          Signals that an I/O exception has occurred.
     * @throws InterruptedException the interrupted exception
     */
    private boolean isActiveViaSc() throws IOException, InterruptedException {
        String output;
        try {
            output = execOutOrFail(scPath().toString(), "query", SERVICE);
            return output.contains("SERVICE_NAME: " + SERVICE);
        } catch (IllegalStateException e) {
            return false;
        }
    }

	/* (non-Javadoc)
     * @see com.perforce.hws.deployment.deployers.Deployer#postUninstall()
     */
    @Override
    public void postUninstall() throws IOException {
        rmtree(getInstallDir(), p -> !p.startsWith(p4ConfigDirPath()));
    }
}
